import click

from context import StacklContext


@click.command()
@click.argument('snapshot_name', required=True)
def restore(snapshot_name):
    stackl_context = StacklContext
    result = stackl_context.snapshot_api.restore_snapshot(snapshot_name)
    click.echo(result)



import os
from pathlib import Path

import click
from context import get_config_path


@click.command()
@click.argument('host', required=True)
def connect(host):
    homedir = str(Path.home())
    if len(homedir) == 0:
        homedir = os.getcwd()
    if not os.path.exists(homedir + os.sep + '.stackl'):
        os.makedirs(homedir + os.sep + '.stackl')
    with open(get_config_path(), 'w+') as stackl_config:
        stackl_config.write(host)






import click
import stackl_client

from context import StacklContext


@click.command()
def info():
    stackl_context = StacklContext()
    about_api = stackl_client.AboutApi(api_client=stackl_context.api_client)
    click.echo(about_api.get_hostname())



import json

import click
import stackl_client
from commands.autocomplete import show_progress_bar
from context import pass_stackl_context, StacklContext


@click.group()
@click.pass_context
def update(ctx):
    ctx.obj = StacklContext()


@update.command()
@click.option('-p', '--params', default=[], multiple=True)
@click.option('-s', '--secrets', default="{}")
@click.option('-d', '--disable-invocation', is_flag=True)
@click.option('-r', '--replicas', default="{}")
@click.option('-s', '--show-progress', default=False, is_flag=True)
@click.argument('instance-name')
@pass_stackl_context
def instance(stackl_context: StacklContext, params, secrets, replicas, show_progress,
             disable_invocation, instance_name):
    final_params = {}
    for item in params:
        final_params = {**final_params, **json.loads(item)}
    invocation = stackl_client.StackInstanceUpdate(
        stack_instance_name=instance_name,
        params=final_params,
        secrets=json.loads(secrets),
        replicas=json.loads(replicas))
    if disable_invocation:
        invocation.disable_invocation = True
    res = stackl_context.stack_instances_api.put_stack_instance(invocation)
    click.echo(res)
    if show_progress:
        show_progress_bar(stackl_context, instance_name)


@update.command()
@click.option('-p', '--params', default="{}")
@click.option('-i', '--infrastructure-target', required=True)
@click.option('-s', '--service', required=True)
@click.argument('instance-name')
@pass_stackl_context
def outputs(stackl_context: StacklContext, params, infrastructure_target, service, instance_name):
    outputs_update = stackl_client.OutputsUpdate(outputs=json.loads(params), infrastructure_target=infrastructure_target,
                                                 service=service, stack_instance=instance_name)
    stackl_context.outputs_api.add_outputs(outputs_update)
    click.echo("Updated outputs")



import click

from context import pass_stackl_context, StacklContext


@click.group()
@click.pass_context
def delete(ctx):
    ctx.obj = StacklContext()


@delete.command()
@click.argument('instance-name')
@click.option('--force', default=False, is_flag=True)
@pass_stackl_context
def instance(stackl_context: StacklContext, instance_name, force):
    res = stackl_context.stack_instances_api.delete_stack_instance(
        instance_name, force=force)
    click.echo(res)


@delete.command()
@click.argument('snapshot-name')
@pass_stackl_context
def snapshot(stackl_context: StacklContext, snapshot_name):
    res = stackl_context.snapshot_api.delete_snapshot(snapshot_name)
    click.echo(res)


@delete.command()
@click.argument('environment-name')
@pass_stackl_context
def environment(stackl_context: StacklContext, environment_name):
    res = stackl_context.infrastructure_base_api.delete_infrastructure_base(
        "environment", environment_name)
    click.echo(res)


@delete.command()
@click.argument('location-name')
@pass_stackl_context
def location(stackl_context: StacklContext, location_name):
    res = stackl_context.infrastructure_base_api.delete_infrastructure_base(
        "location", location_name)
    click.echo(res)


@delete.command()
@click.argument('zone-name')
@pass_stackl_context
def zone(stackl_context: StacklContext, zone_name):
    res = stackl_context.infrastructure_base_api.delete_infrastructure_base(
        "zone", zone_name)
    click.echo(res)


@delete.command()
@click.argument('sat-name')
@pass_stackl_context
def sat(stackl_context: StacklContext, sat_name):
    res = stackl_context.sat_api.delete_stack_application_template(sat_name)
    click.echo(res)


@delete.command()
@click.argument('sit-name')
@pass_stackl_context
def sit(stackl_context: StacklContext, sit_name):
    res = stackl_context.sit_api.delete_stack_infrastructure_template(sit_name)
    click.echo(res)


@delete.command()
@click.argument('service-name')
@pass_stackl_context
def service(stackl_context: StacklContext, service_name):
    res = stackl_context.services_api.delete_service(service_name)
    click.echo(res)


@delete.command()
@click.argument('functional-requirement-name')
@pass_stackl_context
def functional_requirement(stackl_context: StacklContext,
                           functional_requirement_name):
    res = stackl_context.functional_requirements_api.delete_functional_requirement(
        functional_requirement_name)
    click.echo(res)


@delete.command()
@click.argument('policy-template-name')
@pass_stackl_context
def policy_template(stackl_context: StacklContext, policy_template_name):
    res = stackl_context.policy_templates_api.delete_policy_template(
        policy_template_name)
    click.echo(res)



import json

import click
import yaml
from commands.autocomplete import get_stack_instances, get_environments, get_locations, get_zones, get_sats, get_sits, \
    get_services, get_functional_requirements, get_policy_templates
from stackl_client import StackInfrastructureTemplate, ApiException
from tabulate import tabulate

try:
    from yaml import CDumper as Dumper, SafeDumper
except ImportError:
    from yaml import Dumper

from context import pass_stackl_context, StacklContext


@click.group()
@click.pass_context
def get(ctx):
    ctx.obj = StacklContext()


def table_data(stackl_object):
    table = []
    if isinstance(stackl_object, list):
        if stackl_object and isinstance(stackl_object[0],
                                        StackInfrastructureTemplate):
            for so in stackl_object:
                table.append([so.name, "", "", ""])
                for it in so.infrastructure_targets:
                    table.append(["", it.environment, it.location, it.zone])
                return table, ["name", "environment", "location", "zone"]
        else:
            for so in stackl_object:
                table.append([so.name])
            return table, ["name"]
    else:
        if isinstance(stackl_object, StackInfrastructureTemplate):
            table.append([stackl_object.name, "", "", ""])
            for it in stackl_object.infrastructure_targets:
                table.append(["", it.environment, it.location, it.zone])
            return table, ["name", "environment", "location", "zone"]
        else:
            table.append([stackl_object.name])
            return table, ["name"]


def parse(stackl_object, output_type):
    if output_type is None:
        data, headers = table_data(stackl_object)
        return tabulate(data, headers)
    else:
        if isinstance(stackl_object, list):
            table = []
            for so in stackl_object:
                table.append(so.to_dict())
            dump_object = table
        else:
            dump_object = stackl_object.to_dict()
        if output_type == 'yaml':
            return yaml.dump(dump_object, Dumper=Dumper)
        elif output_type == 'json':
            return json.dumps(dump_object,
                              sort_keys=True,
                              indent=4,
                              separators=(',', ': '))
        else:
            return "Output type not supported"



@get.command()
@click.option('-o', '--output')
@click.argument('name', required=False, autocompletion=get_stack_instances)
@pass_stackl_context
def instance(stackl_context: StacklContext, output, name):
    try:
        if name is None:
            env = stackl_context.stack_instances_api.get_stack_instances()
        else:
            env = stackl_context.stack_instances_api.get_stack_instance(name)
        click.echo(parse(env, output))
    except ApiException as e:
        click.echo(e.body)


@get.command()
@click.option('-o', '--output')
@click.argument('name', required=False, autocompletion=get_environments)
@pass_stackl_context
def environment(stackl_context: StacklContext, output, name):
    document_type = "environment"
    try:
        if name is None:
            env = stackl_context.infrastructure_base_api.get_infrastructure_base_by_type(
                document_type)
        else:
            env = stackl_context.infrastructure_base_api.get_infrastructure_base_by_type_and_name(
                document_type, name)
        click.echo(parse(env, output))
    except ApiException as e:
        click.echo(e.body)


@get.command()
@click.option('-o', '--output')
@click.argument('name', required=False, autocompletion=get_locations)
@pass_stackl_context
def location(stackl_context: StacklContext, output, name):
    document_type = "location"
    try:
        if name is None:
            env = stackl_context.infrastructure_base_api.get_infrastructure_base_by_type(
                document_type)
        else:
            env = stackl_context.infrastructure_base_api.get_infrastructure_base_by_type_and_name(
                document_type, name)
        click.echo(parse(env, output))
    except ApiException as e:
        click.echo(e.body)


@get.command()
@click.option('-o', '--output')
@click.argument('name', required=False, autocompletion=get_zones)
@pass_stackl_context
def zone(stackl_context: StacklContext, output, name):
    document_type = "zone"
    try:
        if name is None:
            env = stackl_context.infrastructure_base_api.get_infrastructure_base_by_type(
                document_type)
        else:
            env = stackl_context.infrastructure_base_api.get_infrastructure_base_by_type_and_name(
                document_type, name)
        click.echo(parse(env, output))
    except ApiException as e:
        click.echo(e.body)


@get.command()
@click.option('-o', '--output')
@click.argument('name', required=False, autocompletion=get_sats)
@pass_stackl_context
def sat(stackl_context: StacklContext, output, name):
    try:
        if name is None:
            sat = stackl_context.sat_api.get_stack_application_templates()
        else:
            sat = stackl_context.sat_api.get_stack_application_template_by_name(
                name)
        click.echo(parse(sat, output))
    except ApiException as e:
        click.echo(e.body)


@get.command()
@click.option('-o', '--output')
@click.argument('name', required=False, autocompletion=get_sits)
@pass_stackl_context
def sit(stackl_context: StacklContext, output, name):
    try:
        if name is None:
            sit = stackl_context.sit_api.get_stack_infrastructure_templates()
        else:
            sit = stackl_context.sit_api.get_stack_infrastructure_template_by_name(
                name)
        click.echo(parse(sit, output))
    except ApiException as e:
        click.echo(e.body)


@get.command()
@click.option('-o', '--output')
@click.argument('name', required=False, autocompletion=get_services)
@pass_stackl_context
def service(stackl_context: StacklContext, output, name):
    try:
        if name is None:
            env = stackl_context.services_api.get_services()
        else:
            env = stackl_context.services_api.get_service_by_name(name)
        click.echo(parse(env, output))
    except ApiException as e:
        click.echo(e.body)


@get.command()
@click.option('-o', '--output')
@click.argument('name', required=False, autocompletion=get_functional_requirements)
@pass_stackl_context
def functional_requirement(stackl_context: StacklContext, output, name):
    try:
        if name is None:
            env = stackl_context.functional_requirements_api.get_functional_requirements(
            )
        else:
            env = stackl_context.functional_requirements_api.get_functional_requirement_by_name(
                name)
        click.echo(parse(env, output))
    except ApiException as e:
        click.echo(e.body)


@get.command()
@click.option('-o', '--output')
@click.argument('name', required=False, autocompletion=get_policy_templates)
@pass_stackl_context
def policy_template(stackl_context: StacklContext, output, name):
    try:
        if name is None:
            env = stackl_context.policy_templates_api.get_policy_templates()
        else:
            env = stackl_context.policy_templates_api.get_policy_template_by_name(
                name)
        click.echo(parse(env, output))
    except ApiException as e:
        click.echo(e.body)


@get.command()
@click.option('-o', '--output')
@click.argument('type')
@click.argument('name')
@pass_stackl_context
def snapshots(stackl_context: StacklContext, type, name, output):
    try:
        snapshots = stackl_context.snapshot_api.get_snapshots(type, name)
        click.echo(parse(snapshots, output))
    except ApiException as e:
        click.echo(e.body)


@get.command()
@click.option('-o', '--output')
@click.argument("name")
@pass_stackl_context
def snapshot(stackl_context: StacklContext, name, output):
    try:
        snapshot = stackl_context.snapshot_api.get_snapshot(name)
        click.echo(parse(snapshot, output))
    except ApiException as e:
        click.echo(e.body)



import click
import yaml
from commands.autocomplete import get_zones, get_locations, get_environments, get_services, get_functional_requirements, \
    get_policy_templates, get_sits, get_sats

from context import pass_stackl_context, StacklContext

try:
    from yaml import CDumper as Dumper
except ImportError:
    from yaml import Dumper


@click.group()
@click.pass_context
def edit(ctx):
    ctx.obj = StacklContext()


@edit.command()
@click.argument('name', autocompletion=get_zones)
@pass_stackl_context
def zone(stackl_context: StacklContext, name):
    zone = stackl_context.infrastructure_base_api.get_infrastructure_base_by_type_and_name(
        "zone", name)
    new_zone = click.edit(yaml.dump(zone.to_dict(), Dumper=Dumper))
    if new_zone is not None:
        new_zone = yaml.load(new_zone, Loader=yaml.FullLoader)
        stackl_context.infrastructure_base_api.put_infrastructure_base(
            new_zone)
        click.echo("Zone updated")
    else:
        click.echo("No changes, document not modified")


@edit.command()
@click.argument('name', autocompletion=get_locations)
@pass_stackl_context
def location(stackl_context: StacklContext, name):
    location = stackl_context.infrastructure_base_api.get_infrastructure_base_by_type_and_name(
        "location", name)
    new_location = click.edit(yaml.dump(location.to_dict(), Dumper=Dumper))
    if new_location is not None:
        new_location = yaml.load(new_location, Loader=yaml.FullLoader)
        stackl_context.infrastructure_base_api.put_infrastructure_base(
            new_location)
        click.echo("Location updated")
    else:
        click.echo("No changes, document not modified")


@edit.command()
@click.argument('name', autocompletion=get_environments)
@pass_stackl_context
def environment(stackl_context: StacklContext, name):
    environment = stackl_context.infrastructure_base_api.get_infrastructure_base_by_type_and_name(
        "environment", name)
    new_environment = click.edit(
        yaml.dump(environment.to_dict(), Dumper=Dumper))
    if new_environment is not None:
        new_environment = yaml.load(new_environment, Loader=yaml.FullLoader)
        stackl_context.infrastructure_base_api.put_infrastructure_base(
            new_environment)
        click.echo("Environment updated")
    else:
        click.echo("No changes, document not modified")


@edit.command()
@click.argument('name', autocompletion=get_services)
@pass_stackl_context
def service(stackl_context: StacklContext, name):
    service = stackl_context.services_api.get_service_by_name(name)
    new_service = click.edit(yaml.dump(service.to_dict(), Dumper=Dumper))
    if new_service is not None:
        new_service = yaml.load(new_service, Loader=yaml.FullLoader)
        stackl_context.services_api.put_service(new_service)
        click.echo("Service  updated")
    else:
        click.echo("No changes, document not modified")


@edit.command()
@click.argument('name', autocompletion=get_functional_requirements)
@pass_stackl_context
def functional_requirement(stackl_context: StacklContext, name):
    functional_requirement = stackl_context.functional_requirements_api.get_functional_requirement_by_name(name)
    new_functional_requirement = click.edit(yaml.dump(functional_requirement.to_dict(), Dumper=Dumper))
    if new_functional_requirement is not None:
        new_functional_requirement = yaml.load(new_functional_requirement, Loader=yaml.FullLoader)
        stackl_context.functional_requirements_api.put_functional_requirement(new_functional_requirement)
        click.echo("Functional Requirement updated")
    else:
        click.echo("No changes, document not modified")


@edit.command()
@click.argument('name', autocompletion=get_policy_templates)
@pass_stackl_context
def policy_template(stackl_context: StacklContext, name):
    policy_template = stackl_context.policy_templates_api.get_policy_template_by_name(name)
    new_policy_template = click.edit(yaml.dump(policy_template.to_dict(), Dumper=Dumper))
    if new_policy_template is not None:
        new_policy_template = yaml.load(new_policy_template, Loader=yaml.FullLoader)
        stackl_context.policy_templates_api.put_policy_template(new_policy_template)
        click.echo("Policy template updated")
    else:
        click.echo("No changes, document not modified")


@edit.command()
@click.argument('name', autocompletion=get_sits)
@pass_stackl_context
def sit(stackl_context: StacklContext, name):
    sit = stackl_context.sit_api.get_stack_infrastructure_template_by_name(
        name)
    new_sit = click.edit(yaml.dump(sit.to_dict(), Dumper=Dumper))
    if new_sit is not None:
        new_sit = yaml.load(new_sit, Loader=yaml.FullLoader)
        stackl_context.infrastructure_base_api.put_infrastructure_base(new_sit)
        click.echo("SIT updated")
    else:
        click.echo("No changes, document not modified")


@edit.command()
@click.argument('name', autocompletion=get_sats)
@pass_stackl_context
def sat(stackl_context: StacklContext, name):
    sat = stackl_context.sat_api.get_stack_infrastructure_template_by_name(
        name)
    new_sat = click.edit(yaml.dump(sat.to_dict(), Dumper=Dumper))
    if new_sat is not None:
        new_sat = yaml.load(new_sat, Loader=yaml.FullLoader)
        stackl_context.infrastructure_base_api.put_infrastructure_base(new_sat)
        click.echo("SAT updated")
    else:
        click.echo("No changes, document not modified")



import json
import time

import click
import stackl_client
from commands.autocomplete import show_progress_bar
from context import pass_stackl_context, StacklContext


@click.group()
@click.pass_context
def create(ctx):
    ctx.obj = StacklContext()


@create.command()
@click.option('--stack-infrastructure-template')
@click.option('--stack-application-template')
@click.option('-p', '--params', default=[], multiple=True)
@click.option('-t', '--tags', default="{}")
@click.option('-r', '--replicas', default="{}")
@click.option('-s', '--show-progress', default=False, is_flag=True)
@click.argument('instance-name')
@pass_stackl_context
def instance(stackl_context: StacklContext, stack_infrastructure_template,
             stack_application_template, params, tags, replicas, show_progress,
             instance_name):
    final_params = {}
    for item in params:
        final_params = {**final_params, **json.loads(item)}
    invocation = stackl_client.StackInstanceInvocation(
        stack_instance_name=instance_name,
        stack_infrastructure_template=stack_infrastructure_template,
        stack_application_template=stack_application_template,
        replicas=json.loads(replicas),
        params=final_params,
        tags=json.loads(tags))
    res = stackl_context.stack_instances_api.post_stack_instance(invocation)
    click.echo(res)
    if show_progress:
        show_progress_bar(stackl_context, instance_name)



@create.command()
@click.argument("type", required=True)
@click.argument("name", required=True)
@pass_stackl_context
def snapshot(stackl_context: StacklContext, type, name):
    result = stackl_context.snapshot_api.create_snapshot(type, name)
    click.echo(result)


@create.command()
@click.option('-p', '--policy-file', type=click.File())
@click.option('-i', '--inputs', type=click.STRING, default="")
@click.argument('policy-name')
@pass_stackl_context
def policy(stackl_context: StacklContext, policy_file, policy_name, inputs):
    inputs = [i.strip() for i in inputs.split(',')]
    policy = stackl_client.Policy(name=policy_name,
                                  type="policy",
                                  category="configs",
                                  description="policy through cli",
                                  policy=policy_file.read(),
                                  inputs=inputs)
    res = stackl_context.policies_api.put_policy(policy)
    click.echo(res)



import os
from pathlib import Path

import click
import stackl_client


class StacklContext(object):
    def __init__(self):
        try:
            with open(get_config_path(), 'r+') as stackl_config:
                host = stackl_config.read()
                configuration = stackl_client.Configuration()
                configuration.host = host
                self.api_client = stackl_client.ApiClient(configuration=configuration)
                self.infrastructure_base_api = stackl_client.InfrastructureBaseApi(
                    api_client=self.api_client)
                self.functional_requirements_api = stackl_client.FunctionalRequirementsApi(
                    api_client=self.api_client)
                self.services_api = stackl_client.ServicesApi(
                    api_client=self.api_client)
                self.sat_api = stackl_client.StackApplicationTemplatesApi(
                    api_client=self.api_client)
                self.sit_api = stackl_client.StackInfrastructureTemplatesApi(
                    api_client=self.api_client)
                self.stack_instances_api = stackl_client.StackInstancesApi(
                    api_client=self.api_client)
                self.policy_templates_api = stackl_client.PolicyTemplatesApi(
                    api_client=self.api_client)
                self.snapshot_api = stackl_client.SnapshotsApi(
                    api_client=self.api_client)
                self.outputs_api = stackl_client.OutputsApi(api_client=self.api_client)
        except FileNotFoundError:
            click.echo(
                "Config file not found, run `stackl connect` first")
            exit(1)


pass_stackl_context = click.make_pass_decorator(StacklContext, ensure=False)


def get_config_path():
    if len(str(Path.home())) == 0:
        config_path = os.getcwd() + os.sep + '.stackl' + os.sep + 'config'
    else:
        config_path = str(Path.home()) + os.sep + '.stackl' + os.sep + 'config'
    return config_path



import time

import click
from context import StacklContext


def get_stackl_context():
    return StacklContext()


def get_stack_instances(ctx, args, incomplete):
    stack_instances = get_stackl_context().stack_instances_api.get_stack_instances()
    return [si.name for si in stack_instances if si.name.startswith(incomplete)]


def get_environments(ctx, args, incomplete):
    environments = get_stackl_context().infrastructure_base_api.get_infrastructure_base_by_type("environment")
    return [env.name for env in environments if env.name.startswith(incomplete)]


def get_locations(ctx, args, incomplete):
    locations = get_stackl_context().infrastructure_base_api.get_infrastructure_base_by_type("location")
    return [loc.name for loc in locations if loc.name.startswith(incomplete)]


def get_zones(ctx, args, incomplete):
    zones = get_stackl_context().infrastructure_base_api.get_infrastructure_base_by_type("zone")
    return [z.name for z in zones if z.name.startswith(incomplete)]


def get_sats(ctx, args, incomplete):
    sats = get_stackl_context().sat_api.get_stack_application_templates()
    return [s.name for s in sats if s.name.startswith(incomplete)]


def get_sits(ctx, args, incomplete):
    sits = get_stackl_context().sit_api.get_stack_infrastructure_templates()
    return [s.name for s in sits if s.name.startswith(incomplete)]


def get_services(ctx, args, incomplete):
    services = get_stackl_context().services_api.get_services()
    return [s.name for s in services if s.name.startswith(incomplete)]


def get_functional_requirements(ctx, args, incomplete):
    frs = get_stackl_context().functional_requirements_api.get_functional_requirements()
    return [s.name for s in frs if s.name.startswith(incomplete)]


def get_policy_templates(ctx, args, incomplete):
    pts = get_stackl_context().policy_templates_api.get_policy_templates()
    return [pt.name for pt in pts if pt.name.startswith(incomplete)]


def show_progress_bar(stackl_context, instance_name):
    stack_instance = stackl_context.stack_instances_api.get_stack_instance(instance_name)
    with click.progressbar(length=len(stack_instance.status),
                           label='Stack Instance Status') as bar:
        bar.update(0)
        ready = False
        while not ready:
            for status in stack_instance.status:
                if status.status == "FAILED":
                    click.echo(
                        f"Stack instance {stack_instance.name}: service {status.service} on functional-requirement {status.functional_requirement} failed!")
                    exit(1)
                elif status.status == "in_progress":
                    time.sleep(1)
                    stack_instance = stackl_context.stack_instances_api.get_stack_instance(instance_name)
                    ready = False
                    break
                else:
                    bar.update(1)
                    ready = True



from setuptools import setup, find_packages

__version__ = "0.2.5"

setup(
    name='stackl-cli',
    version=__version__,
    py_modules=['stackl', 'commands', 'context'],
    packages=find_packages(),
    install_requires=[
        f'stackl-client=={__version__}', 'pyYAML==5.3', 'Click==7.0',
        'mergedeep==1.3.0', 'tabulate==0.8.6', 'glom==19.10.0'
    ],
    entry_points='''
        [console_scripts]
        stackl=stackl:cli
    ''',
)



import click

from commands.apply import apply
from commands.connect import connect
from commands.create import create
from commands.delete import delete
from commands.edit import edit
from commands.get import get
from commands.info import info
from commands.restore import restore
from commands.update import update

CLICK_CONTEXT_SETTINGS = dict(help_option_names=['-h', '--help'])


@click.version_option()
@click.group(context_settings=CLICK_CONTEXT_SETTINGS)
def cli():
    pass


cli.add_command(connect)
cli.add_command(get)
cli.add_command(info)
cli.add_command(apply)
cli.add_command(edit)
cli.add_command(create)
cli.add_command(update)
cli.add_command(delete)
cli.add_command(restore)



import json
from pathlib import Path
from sys import exit

import click
import stackl_client
import yaml
from commands.autocomplete import show_progress_bar
from context import StacklContext
from mergedeep import merge


@click.command()
@click.option('-d', '--directory', type=click.Path(exists=True))
@click.option('-c', '--config-file', type=click.File())
@click.option('-p', '--params', default=[], multiple=True)
@click.option('-t', '--tags', default="{}")
@click.option('-r', '--replicas', default="{}")
@click.option('-s', '--secrets', default="{}")
@click.option('-e', '--service-params', default="{}")
@click.option('-s', '--show-progress', default=False, is_flag=True)
@click.argument('instance-name', required=False)
def apply(directory, config_file, params, tags, secrets,
          service_params, replicas, instance_name, show_progress):
    stackl_context = StacklContext()
    if instance_name is None:
        upload_files(directory, stackl_context)
    else:
        apply_stack_instance(config_file, params, tags, secrets,
                             service_params, replicas, stackl_context,
                             instance_name, show_progress)


def apply_stack_instance(config_file, params, tags, secrets, service_params,
                         replicas, stackl_context, instance_name, show_progress):
    final_params = {}
    for item in params:
        final_params = {**final_params, **json.loads(item)}
    config_doc = yaml.load(config_file.read(), Loader=yaml.FullLoader)
    final_params = {**config_doc['params'], **final_params}
    tags = json.loads(tags)
    replicas = json.loads(replicas)
    if "replicas" in config_doc:
        replicas = {**config_doc['replicas'], **replicas}
    secrets = json.loads(secrets)
    service_params = json.loads(service_params)
    if "service_params" in config_doc:
        service_params = merge(config_doc['service_params'], service_params)
    if "secrets" in config_doc:
        secrets = {**config_doc['secrets'], **secrets}
    if "tags" in config_doc:
        tags = {**config_doc['tags'], **tags}
    invocation = stackl_client.StackInstanceInvocation(
        stack_instance_name=instance_name,
        stack_infrastructure_template=config_doc[
            "stack_infrastructure_template"],
        stack_application_template=config_doc["stack_application_template"],
        params=final_params,
        replicas=replicas,
        service_params=service_params,
        secrets=secrets,
        tags=tags)
    try:
        stackl_context.stack_instances_api.get_stack_instance(instance_name)
        res = stackl_context.stack_instances_api.put_stack_instance(invocation)
    except stackl_client.exceptions.ApiException:
        res = stackl_context.stack_instances_api.post_stack_instance(
            invocation)

    click.echo(res)

    if show_progress:
        show_progress_bar(stackl_context, instance_name)


def upload_files(directory, stackl_context):
    for path in Path(directory).rglob('*.yml'):
        with open(path, 'r') as doc:
            # ignore dotfiles
            if path.name.startswith('.'):
                continue
            click.echo(f"Reading document: {str(path)}")
            stackl_doc = yaml.load(doc.read(), Loader=yaml.FullLoader)
            if 'name' in stackl_doc:
                click.echo(
                    f"Applying stackl document: {str(path) + ' ' + stackl_doc['name']}"
                )
            else:
                click.echo(
                    f"Error in stackl document, no 'name' found: {path}")
            try:
                if stackl_doc["type"] in ["environment", "location", "zone"]:
                    stackl_context.infrastructure_base_api.put_infrastructure_base(
                        stackl_doc)
                if stackl_doc["type"] == "functional_requirement":
                    stackl_context.functional_requirements_api.put_functional_requirement(
                        stackl_doc)
                if stackl_doc["type"] == "service":
                    stackl_context.services_api.put_service(stackl_doc)
                if stackl_doc["type"] == "stack_application_template":
                    stackl_context.sat_api.put_stack_application_template(
                        stackl_doc)
                if stackl_doc["type"] == "stack_infrastructure_template":
                    stackl_context.sit_api.put_stack_infrastructure_template(
                        stackl_doc)
                if stackl_doc["type"] == "policy_template":
                    stackl_context.policy_templates_api.put_policy_template(
                        stackl_doc)
                click.echo(
                    f"Succesfully applied {stackl_doc['name']} with type {stackl_doc['type']}"
                )
            except stackl_client.exceptions.ApiException as e:
                click.echo(
                    f"Failed to apply {stackl_doc['name']} with type {stackl_doc['type']}: {e.body}"
                )
                exit(1)
